#ifndef PCPokktManager_h
#define PCPokktManager_h

#include <stdio.h>
#include "cocos2d.h"
#include "PCPokktConfig.h"
#include "PCAdConfig.h"
#include "IAPDetails.h"
#include "PCParser.h"
#include "PCEnums.h"


namespace pokkt
{
#define POKKTLOG(format, ...) cocos2d::log(format, ##__VA_ARGS__)
    
    typedef void (cocos2d::CCObject::*EVT_SELECTOR_POKKT_INITIALIZED)(bool, std::string);
#define PC_POKKT_INITIALIZED_EVENT_SELECTOR(_SELECTOR) (EVT_SELECTOR_POKKT_INITIALIZED)(&_SELECTOR)
    
    typedef void (cocos2d::CCObject::*EVT_SELECTOR_OFFERWALL)(std::string);
#define PC_OFFERWALL_EVENT_SELECTOR(_SELECTOR) (EVT_SELECTOR_OFFERWALL)(&_SELECTOR)
    
    typedef void (cocos2d::CCObject::*EVT_SELECTOR_AD_EVENT)(PCAdConfig*, std::map<std::string, std::string>);
#define PC_AD_EVENT_SELECTOR(_SELECTOR) (EVT_SELECTOR_AD_EVENT)(&_SELECTOR)
    
    typedef void (cocos2d::CCObject::*EVT_SELECTOR_BANNER)(std::string, std::string);
#define PC_BANNER_EVENT_SELECTOR(_SELECTOR) (EVT_SELECTOR_BANNER)(&_SELECTOR)
    
    class PCPokktManager
    {
    public:
        
        // pokkt standard APIs
        inline static void setDebug(bool value) { notifyNative(SET_DEBUG_OP, value ? "true" : "false"); }
        inline static void exportLog() { notifyNative(EXPORT_LOG_OP); }
        inline static void showToast(std::string message) { notifyNative(SHOW_TOAST_OP, message); }
        inline static void showLog(std::string message) { notifyNative(SHOW_LOG_OP, message); }
        inline static void initPokkt(PCPokktConfig* config) { notifyNative(INIT_POKKT_OP, toPokktConfigJSONString(config)); }
        inline static void updatePokktConfig(PCPokktConfig* config) { notifyNative(UPDATE_POKKT_CONFIG_OP, toPokktConfigJSONString(config)); }
        inline static std::string getSDKVersion() { return getSDKVersionOnNative(); }
        
        // session related APIs
        inline static void startSession() { notifyNative(START_SESSION_OP); }
        inline static void endSession() { notifyNative(END_SESSION_OP); }
        inline static void notifyAppInstall()  { notifyNative(NOTIFY_APP_INSTALL_OP); }
        inline static void trackIAP(IAPDetails* details)  { notifyNative(START_SESSION_OP, toIAPDetailsJSONString(details)); }
        
        
        // offerwall APIs
        inline static void getCoins(PCPokktConfig* config) { notifyNative(GET_COINS_OP, toPokktConfigJSONString(config)); }
        inline static void getPendingCoins() { notifyNative(GET_PENDING_COINS_OP); }
        inline static void checkOfferwallCampaign() { notifyNative(CHECK_OFFERWALL_CAMPAIGN_OP); }
        
        // ad related APIs
        inline static void cacheAd(PCAdConfig* adConfig) { notifyNative(CACHE_AD_OP, toAdConfigJSONString(adConfig)); }
        inline static void showAd(PCAdConfig* adConfig) { notifyNative(SHOW_AD_OP, toAdConfigJSONString(adConfig)); }
        inline static void checkAdAvailability(PCAdConfig* adConfig) { notifyNative(CHECK_AD_AVAILABILITY_OP, toAdConfigJSONString(adConfig)); }
        inline static double getRewardedAdVc(PCAdConfig* adConfig) { return getRewardedAdVc(adConfig); }
        
        inline static void loadBanner(std::string screenName, int bannerPosition){
            notifyNative(LOAD_BANNER_OP, toBannerParamsJSONString(screenName, bannerPosition));
        }
        
        inline static void loadBannerWithRect(std::string screenName, int width, int height, int x, int y){
            notifyNative(LOAD_BANNER_RECT_OP, toBannerRectParamsJSONString(screenName, width, height, x, y));
        }
        
        inline static void setBannerAutoRefresh(bool autoRefresh){ notifyNative(BANNER_AUTO_REFRESH, autoRefresh? "true" : "false");}
        
        inline static void removeBanner(std::string screenName){ notifyNative(REMOVE_BANNER, screenName);}
        
        // handle messages via native (java/obj-c)
        static void handleNativeMessage(std::string operation, std::string param);
        
        // sets a listener for an event (only one listener per event
        static void setPokktInitializedListener(EVT_SELECTOR_POKKT_INITIALIZED listener, cocos2d::CCObject *target);
        static void removePokktInitializedListener();
        
        static void setOfferwallEventListener(std::string eventName, EVT_SELECTOR_OFFERWALL listener, cocos2d::CCObject *target);
        static void removeOfferwallEventListener(std::string eventName);
        
        static void setAdEventListener(std::string eventName, EVT_SELECTOR_AD_EVENT listener, cocos2d::CCObject *target);
        static void removeAdEventListener(std::string eventName);
        
        static void setBannerEventListener(std::string eventName, EVT_SELECTOR_BANNER listener, cocos2d::CCObject *target);
        static void removeBannerEventListener(std::string eventName);
        
        ~PCPokktManager(){}
        
    private:
        
        typedef struct __listenerTargetPairOfferwall
        {
            EVT_SELECTOR_OFFERWALL listener;
            cocos2d::CCObject *target;
        } ListenerTargetPairOfferwall;
        
        typedef struct __listenerTargetPairAdEvent
        {
            EVT_SELECTOR_AD_EVENT listener;
            cocos2d::CCObject *target;
        } ListenerTargetPairAdEvent;
        
        typedef struct __listenerTargetPairBannerEvent
        {
            EVT_SELECTOR_BANNER listener;
            cocos2d::CCObject *target;
        } ListenerTargetPairBannerEvent;
        
        static cocos2d::CCObject* _pokktInitializedEventTarget;
        static EVT_SELECTOR_POKKT_INITIALIZED _pokktInitializedListner;
        static std::map<std::string, ListenerTargetPairOfferwall*> _eventMapOfferwall;
        static std::map<std::string, ListenerTargetPairAdEvent*> _eventMapAdEvent;
        static std::map<std::string, ListenerTargetPairBannerEvent*> _eventMapBannerEvent;
        
        // use this to notify native pokkt runtime
        static void notifyNative(std::string operation, std::string param = "");
        
        // use this to get the coin associated
        static double getRewardedAdVcOnNative(PCAdConfig* adConfig);
        
        // sdk version
        static std::string getSDKVersionOnNative();
    };
}

#endif /* PCPokktManager_h */
